package it.enea.xlab.GECORegistration.controller;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import javax.mail.MessagingException;
import javax.mail.internet.MimeMessage;
import javax.ws.rs.POST;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.mail.SimpleMailMessage;
import org.springframework.mail.javamail.JavaMailSender;
import org.springframework.mail.javamail.MimeMessageHelper;
import org.springframework.ui.freemarker.FreeMarkerTemplateUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.servlet.view.freemarker.FreeMarkerConfigurer;

import freemarker.template.Template;
import freemarker.template.TemplateException;
import io.swagger.annotations.Api;
import it.enea.xlab.GECORegistration.dto.LocalUserDTO;
import it.enea.xlab.GECORegistration.model.user.User;
import it.enea.xlab.GECORegistration.service.user.NewUserInterface;
import lombok.NonNull;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;

@RestController
@Api(value = "UserController")
@Log4j2
@RequiredArgsConstructor
public class MailController {
	
	@NonNull
	private JavaMailSender emailSender;

	//@Autowired
	//private FreeMarkerConfigurer freemarkerConfigurer;

	@NonNull
	NewUserInterface userInterface;

	@Value("${activationLink}")
	private String activationLink;

		
	@POST
	@PostMapping("/user/sendMailMassage")
	public void sendMessageUsingFreemarkerTemplate(LocalUserDTO user)
			throws IOException, TemplateException, MessagingException {

		Map<String, Object> templateModel1 = new HashMap<String, Object>();

		templateModel1.put("link", activationLink + user.getSubject());
		templateModel1.put("recipientName", user.getName() + " " + user.getSurname());
		try {
			String htmlBody = FreeMarkerUtils.getHtml("mailMassage_it_IT.ftl", templateModel1);
			sendHtmlMessage(user.getEmail(), "Activate your registration to SmartEnergyCommunity", htmlBody);
		}catch (Exception e) {
			log.error("Error message:" + e.getLocalizedMessage(), e);
		}

	}

	private void sendHtmlMessage(String to, String subject, String htmlBody) throws MessagingException {
		MimeMessage message = emailSender.createMimeMessage();
		MimeMessageHelper helper = new MimeMessageHelper(message, true, "UTF-8");
		helper.setTo(to);
		helper.setSubject(subject);
		helper.setText(htmlBody, true);
		helper.setFrom("gianluca.dagosta@enea.it");
		emailSender.send(message);
	}

	@POST
	@PostMapping("/user/activate")
	public Boolean activateUser(String subject) throws IOException, TemplateException, MessagingException {

		User user = userInterface.findBySubject(subject);
		user.setActive(true);
		userInterface.save(user);
		if (user.getActive() == true) {
			return true;
		} else
			return false;
	}

}
